@@ -367,22 +367,6 @@ typedef uint16_t	in_port_t;
 # endif /* WORDS_BIGENDIAN */
 #endif /* BYTE_ORDER */
 
-/* Function replacement / compatibility hacks */
-
-#if defined(BROKEN_GETADDRINFO) && defined(HAVE_GETADDRINFO)
-# undef HAVE_GETADDRINFO
-#endif
-#if defined(BROKEN_GETADDRINFO) && defined(HAVE_FREEADDRINFO)
-# undef HAVE_FREEADDRINFO
-#endif
-#if defined(BROKEN_GETADDRINFO) && defined(HAVE_GAI_STRERROR)
-# undef HAVE_GAI_STRERROR
-#endif
-
-#if !defined(HAVE_MEMMOVE) && defined(HAVE_BCOPY)
-# define memmove(s1, s2, n) bcopy((s2), (s1), (n))
-#endif /* !defined(HAVE_MEMMOVE) && defined(HAVE_BCOPY) */
-
 #if !defined(HAVE___func__) && defined(HAVE___FUNCTION__)
 #  define __func__ __FUNCTION__
 #elif !defined(HAVE___func__)
@@ -492,13 +476,6 @@ typedef uint16_t	in_port_t;
 #define	SCOPE_DELIMITER '%'
 #endif
 
-#ifndef HAVE_FLOCK
-#define LOCK_SH         0x01            /* shared file lock */
-#define LOCK_EX         0x02            /* exclusive file lock */
-#define LOCK_NB         0x04            /* don't block when locking */
-#define LOCK_UN         0x08            /* unlock file */
-#endif
-
 #if !HAVE_DECL_LOG_PERROR
 #define LOG_PERROR 0
 #endif
