@@ -1,4 +1,4 @@
-/*	$OpenBSD: control.c,v 1.129 2023/03/08 04:43:15 guenther Exp $	*/
+/*	$OpenBSD: control.c,v 1.130 2023/05/31 16:51:46 op Exp $	*/
 
 /*
  * Copyright (c) 2012 Gilles Chehade <gilles@poolp.org>
@@ -292,13 +292,8 @@ control_accept(int listenfd, short event, void *arg)
 	uid_t			 euid;
 	gid_t			 egid;
 
-#if defined(HAVE_GETDTABLESIZE) && defined(HAVE_GETDTABLECOUNT)
 	if (getdtablesize() - getdtablecount() < CONTROL_FD_RESERVE)
 		goto pause;
-#else
-	if (available_fds(CONTROL_FD_RESERVE))
-		goto pause;
-#endif
 
 	len = sizeof(s_un);
 	if ((connfd = accept(listenfd, (struct sockaddr *)&s_un, &len)) == -1) {
@@ -371,13 +366,8 @@ control_close(struct ctl_conn *c)
 
 	stat_backend->decrement("control.session", 1);
 
-#if defined(HAVE_GETDTABLESIZE) && defined(HAVE_GETDTABLECOUNT)
 	if (getdtablesize() - getdtablecount() < CONTROL_FD_RESERVE)
 		return;
-#else
-	if (available_fds(CONTROL_FD_RESERVE))
-		return;
-#endif
 
 	if (!event_pending(&control_state.ev, EV_READ, NULL)) {
 		log_warnx("warn: re-enabling ctl connections");
