@@ -1,4 +1,4 @@
-/*	$OpenBSD: lka_filter.c,v 1.70 2023/05/15 12:03:04 op Exp $	*/
+/*	$OpenBSD: lka_filter.c,v 1.73 2023/07/07 14:52:00 op Exp $	*/
 
 /*
  * Copyright (c) 2018 Gilles Chehade <gilles@poolp.org>
@@ -33,7 +33,7 @@
 #include "smtpd.h"
 #include "log.h"
 
-#define	PROTOCOL_VERSION	"0.6"
+#define	PROTOCOL_VERSION	"0.7"
 
 struct filter;
 struct filter_session;
@@ -602,40 +602,29 @@ lka_filter_process_response(const char *name, const char *line)
 {
 	uint64_t reqid;
 	uint64_t token;
-	char buffer[LINE_MAX];
 	char *ep = NULL;
-	char *kind = NULL;
-	char *qid = NULL;
-	/*char *phase = NULL;*/
-	char *response = NULL;
-	char *parameter = NULL;
+	const char *kind = NULL;
+	const char *qid = NULL;
+	const char *response = NULL;
+	const char *parameter = NULL;
 	struct filter_session *fs;
 
-	(void)strlcpy(buffer, line, sizeof buffer);
-	if ((ep = strchr(buffer, '|')) == NULL)
+	kind = line;
+
+	if ((ep = strchr(kind, '|')) == NULL)
 		fatalx("Missing token: %s", line);
-	ep[0] = '\0';
-
-	kind = buffer;
-
 	qid = ep+1;
-	if ((ep = strchr(qid, '|')) == NULL)
-		fatalx("Missing reqid: %s", line);
-	ep[0] = '\0';
 
+	errno = 0;
 	reqid = strtoull(qid, &ep, 16);
-	if (qid[0] == '\0' || *ep != '\0')
+	if (qid[0] == '\0' || *ep != '|')
 		fatalx("Invalid reqid: %s", line);
 	if (errno == ERANGE && reqid == ULLONG_MAX)
 		fatal("Invalid reqid: %s", line);
 
-	qid = ep+1;
-	if ((ep = strchr(qid, '|')) == NULL)
-		fatal("Missing directive: %s", line);
-	ep[0] = '\0';
-
+	qid = ep + 1;
 	token = strtoull(qid, &ep, 16);
-	if (qid[0] == '\0' || *ep != '\0')
+	if (qid[0] == '\0' || *ep != '|')
 		fatalx("Invalid token: %s", line);
 	if (errno == ERANGE && token == ULLONG_MAX)
 		fatal("Invalid token: %s", line);
@@ -646,7 +635,7 @@ lka_filter_process_response(const char *name, const char *line)
 	if ((fs = tree_get(&sessions, reqid)) == NULL)
 		return;
 
-	if (strcmp(kind, "filter-dataline") == 0) {
+	if (strncmp(kind, "filter-dataline|", 16) == 0) {
 		if (fs->phase != FILTER_DATA_LINE)
 			fatalx("filter-dataline out of dataline phase");
 		filter_data_next(token, reqid, response);
@@ -655,19 +644,13 @@ lka_filter_process_response(const char *name, const char *line)
 	if (fs->phase == FILTER_DATA_LINE)
 		fatalx("filter-result in dataline phase");
 
-	if ((ep = strchr(response, '|'))) {
+	if ((ep = strchr(response, '|')) != NULL)
 		parameter = ep + 1;
-		ep[0] = '\0';
-	}
 
 	if (strcmp(response, "proceed") == 0) {
-		if (parameter != NULL)
-			fatalx("Unexpected parameter after proceed: %s", line);
 		filter_protocol_next(token, reqid, 0);
 		return;
 	} else if (strcmp(response, "junk") == 0) {
-		if (parameter != NULL)
-			fatalx("Unexpected parameter after junk: %s", line);
 		if (fs->phase == FILTER_COMMIT)
 			fatalx("filter-reponse junk after DATA");
 		filter_result_junk(reqid);
@@ -676,11 +659,11 @@ lka_filter_process_response(const char *name, const char *line)
 		if (parameter == NULL)
 			fatalx("Missing parameter: %s", line);
 
-		if (strcmp(response, "rewrite") == 0)
+		if (strncmp(response, "rewrite|", 8) == 0)
 			filter_result_rewrite(reqid, parameter);
-		else if (strcmp(response, "reject") == 0)
+		else if (strncmp(response, "reject|", 7) == 0)
 			filter_result_reject(reqid, parameter);
-		else if (strcmp(response, "disconnect") == 0)
+		else if (strncmp(response, "disconnect|", 11) == 0)
 			filter_result_disconnect(reqid, parameter);
 		else
 			fatalx("Invalid directive: %s", line);
@@ -942,13 +925,13 @@ filter_protocol_query(struct filter *filter, uint64_t token, uint64_t reqid, con
 		n = io_printf(lka_proc_get_io(filter->proc),
 		    "filter|%s|%lld.%06ld|smtp-in|%s|%016"PRIx64"|%016"PRIx64"|%s|%s\n",
 		    PROTOCOL_VERSION,
-		    (long long)tv.tv_sec, tv.tv_usec,
+		    (long long)tv.tv_sec, (long)tv.tv_usec,
 		    phase, reqid, token, fs->rdns, param);
 	else
 		n = io_printf(lka_proc_get_io(filter->proc),
 		    "filter|%s|%lld.%06ld|smtp-in|%s|%016"PRIx64"|%016"PRIx64"|%s\n",
 		    PROTOCOL_VERSION,
-		    (long long)tv.tv_sec, tv.tv_usec,
+		    (long long)tv.tv_sec, (long)tv.tv_usec,
 		    phase, reqid, token, param);
 	if (n == -1)
 		fatalx("failed to write to processor");
@@ -966,7 +949,7 @@ filter_data_query(struct filter *filter, uint64_t token, uint64_t reqid, const c
 	    "filter|%s|%lld.%06ld|smtp-in|data-line|"
 	    "%016"PRIx64"|%016"PRIx64"|%s\n",
 	    PROTOCOL_VERSION,
-	    (long long)tv.tv_sec, tv.tv_usec,
+	    (long long)tv.tv_sec, (long)tv.tv_usec,
 	    reqid, token, line);
 	if (n == -1)
 		fatalx("failed to write to processor");
@@ -1383,7 +1366,7 @@ report_smtp_broadcast(uint64_t reqid, const char *direction, struct timeval *tv,
 		va_start(ap, format);
 		if (io_printf(lka_proc_get_io(rp->name),
 		    "report|%s|%lld.%06ld|%s|%s|%016"PRIx64"%s",
-		    PROTOCOL_VERSION, (long long)tv->tv_sec, tv->tv_usec,
+		    PROTOCOL_VERSION, (long long)tv->tv_sec, (long)tv->tv_usec,
 		    direction, event, reqid,
 		    format[0] != '\n' ? "|" : "") == -1 ||
 		    io_vprintf(lka_proc_get_io(rp->name), format, ap) == -1)
@@ -1470,7 +1453,7 @@ lka_report_smtp_link_auth(const char *direction, struct timeval *tv, uint64_t re
 		fs->username = xstrdup(username);
 	}
 	report_smtp_broadcast(reqid, direction, tv, "link-auth", "%s|%s\n",
-	    username, result);
+	    result, username);
 }
 
 void
