@@ -1,4 +1,4 @@
-/*	$OpenBSD: smtp_session.c,v 1.433 2022/10/20 01:16:04 millert Exp $	*/
+/*	$OpenBSD: smtp_session.c,v 1.436 2023/10/09 20:55:32 millert Exp $	*/
 
 /*
  * Copyright (c) 2008 Gilles Chehade <gilles@poolp.org>
@@ -29,6 +29,7 @@
 
 #include <netinet/in.h>
 
+#include <asr.h>		/* for asr_freeaddrinfo() */
 #include <ctype.h>
 #include <errno.h>
 #include <inttypes.h>
@@ -226,6 +227,7 @@ static int  smtp_check_starttls(struct smtp_session *, const char *);
 static int  smtp_check_mail_from(struct smtp_session *, const char *);
 static int  smtp_check_rcpt_to(struct smtp_session *, const char *);
 static int  smtp_check_data(struct smtp_session *, const char *);
+static int  smtp_check_noop(struct smtp_session *, const char *);
 static int  smtp_check_noparam(struct smtp_session *, const char *);
 
 static void smtp_filter_phase(enum filter_phase, struct smtp_session *, const char *);
@@ -273,10 +275,6 @@ static void smtp_report_filter_response(struct smtp_session *, int, int, const c
 static void smtp_report_timeout(struct smtp_session *);
 
 
-/* musl work-around */
-void portable_freeaddrinfo(struct addrinfo *);
-
-
 static struct {
 	int code;
 	enum filter_phase filter_phase;
@@ -294,7 +292,7 @@ static struct {
 	{ CMD_DATA,             FILTER_DATA,            "DATA",         smtp_check_data,        smtp_proceed_data },
 	{ CMD_RSET,             FILTER_RSET,            "RSET",         smtp_check_rset,        smtp_proceed_rset },
 	{ CMD_QUIT,             FILTER_QUIT,            "QUIT",         smtp_check_noparam,     smtp_proceed_quit },
-	{ CMD_NOOP,             FILTER_NOOP,            "NOOP",         smtp_check_noparam,     smtp_proceed_noop },
+	{ CMD_NOOP,             FILTER_NOOP,            "NOOP",         smtp_check_noop,        smtp_proceed_noop },
 	{ CMD_HELP,             FILTER_HELP,            "HELP",         smtp_check_noparam,     smtp_proceed_help },
 	{ CMD_WIZ,              FILTER_WIZ,             "WIZ",          smtp_check_noparam,     smtp_proceed_wiz },
 	{ CMD_COMMIT,  		FILTER_COMMIT,		".",		smtp_check_noparam,	smtp_proceed_commit },
@@ -702,7 +700,7 @@ smtp_getaddrinfo_cb(void *arg, int gaierrno, struct addrinfo *ai0)
 				break;
 			}
 		}
-		portable_freeaddrinfo(ai0);
+		asr_freeaddrinfo(ai0);
 	}
 
 	smtp_lookup_servername(s);
@@ -1361,8 +1359,8 @@ smtp_command(struct smtp_session *s, char *line)
 		break;
 
 	case CMD_NOOP:
-		if (!smtp_check_noparam(s, args))
-			break;		
+		if (!smtp_check_noop(s, args))
+			break;
 		smtp_filter_phase(FILTER_NOOP, s, NULL);
 		break;
 
@@ -1648,6 +1646,12 @@ smtp_check_data(struct smtp_session *s, const char *args)
 	return 1;
 }
 
+static int
+smtp_check_noop(struct smtp_session *s, const char *args)
+{
+	return 1;
+}
+
 static int
 smtp_check_noparam(struct smtp_session *s, const char *args)
 {
@@ -2290,7 +2294,7 @@ smtp_auth_failure_pause(struct smtp_session *s)
 	tv.tv_sec = 0;
 	tv.tv_usec = arc4random_uniform(1000000);
 	log_trace(TRACE_SMTP, "smtp: timing-attack protection triggered, "
-	    "will defer answer for %lu microseconds", tv.tv_usec);
+	    "will defer answer for %lu microseconds", (long)tv.tv_usec);
 	evtimer_set(&s->pause, smtp_auth_failure_resume, s);
 	evtimer_add(&s->pause, &tv);
 }
@@ -2635,7 +2639,7 @@ smtp_tx_dataline(struct smtp_tx *tx, const char *line)
 
 		case RFC5322_END_OF_HEADERS:
 			if (tx->session->listener->local ||
-			    tx->session->listener->port == 587) {
+			    tx->session->listener->port == htons(587)) {
 
 				if (!tx->has_date) {
 					log_debug("debug: %p: adding Date", tx);
