@@ -1,4 +1,4 @@
-/*	$OpenBSD: smtpctl.c,v 1.171 2023/05/15 12:03:04 op Exp $	*/
+/*	$OpenBSD: smtpctl.c,v 1.172 2023/05/31 16:51:46 op Exp $	*/
 
 /*
  * Copyright (c) 2013 Eric Faurot <eric@openbsd.org>
@@ -1061,7 +1061,7 @@ main(int argc, char **argv)
 	int		 privileged;
 	char		*argv_mailq[] = { "show", "queue", NULL };
 
-#ifdef NEED_PROGNAME
+#ifndef HAVE___PROGNAME
 	__progname = get_progname(argv[0]);
 #endif
 
